/* -*-c++-*-
 * objects.h --type definitions for primitives and nonprimitives.

 (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

 this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
 <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
 GNU General Public License, it has no warranty what so ever. For more
 details: see the file "COPYING"

 */


#ifndef LIGHT_H
#define LIGHT_H

#include "color.hh"
#include "shapeheads.hh"
#include "object.hh"
#include "ibuf.hh"
#include "proto.hh"


struct illumination_data : shape_data {

    illumination_data();
    ~illumination_data();
    void print() const ;

    void bound(matrix *,bounding_box *);
    bool all_intersections(ibuf * q, ray * r, int flags, bool &b) const;


    vector org;                /* the place where the light is. */
    vector dir;

    color   lcolor;             /* color of the light */


    flt     attenuation_exp,    /* exponent E for intensity = 1/ldist^E */
    tightness,          /* for spotlights */
    radius;             /* radius, used for falloff and penumbra */


    void rotate(const matrix & rotmat);
    void translate(const vector t);
    void scale(const vector s);   
};

/* light source info */
class light_data : public  illumination_data {
    lightbuf *buf;
    static     shape_stats *stat;
    int shad_test,shaded;
public:
    light_data();
    ~light_data();
    
    shape_data *clone () { return new light_data (*this); }    
    shape_stats *getstatp()const { return stat; }
    
    color   get_color(vector lighted_point) const;    
    bool    test(ray *r);
    flt	hitratio();
    void precompute();
    
    static void  print_list();
    static int count();    
};


struct fog_data : illumination_data {
    void precompute();
    shape_data *clone ()  { return new fog_data (*this); }
    static shape_stats *stat;
    shape_stats *getstatp()const { return stat; }

    static void  print_list();
    static color get_color(ray *r, flt end);
};

int light_list_len();
class light_iter {
    int j;
public:
    light_iter();
    bool ok();
    void next();
    light_data *data();
    object *operator->();
};
extern flt light_eps;
PUBLIC object *get_new_fog_object();
PUBLIC object *get_new_light_object();

#endif




