/*
  lightbuf.hh --type definitions for shadow caching.

  (c)  1995,'96 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

  this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
  <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
  GNU General Public License, it has no warranty what so ever. For more
  details: see the file "COPYING"

 */

#ifndef LIGHTBUF_HH
#define LIGHTBUF_HH
#include  "ray.hh"
#include "vray.hh"

extern int lightbufres;
typedef svec<object *> lb_list;			  // list of objects which shadow

class lightbuf_pixel {

    lb_list shadowers;				  // potential shadowers
public:    
    bool test(ray *r)const;			  // test if ray intersects a shadower.
    void print() const;
};

class lightbuf;

/// one plane from ligthbuffer cube   
class lightbuf_plane  {
    lightbuf_pixel **pixels;			  // square array of pixels, size lightbufres
    axes axis;			// axis of normal
//    vector n;			// normalvector to the plane.

 public:
     lightbuf_pixel *find_pixel(vector x) const;
     lightbuf_plane();
     ~lightbuf_plane();
    void init(vector);
    void print() const;
};
/**
  one plane from #lightbuf#, its normal points in direction #n#.
  */


/// a cube centered around lightsource
class lightbuf {
    lightbuf_plane faces[6];			  // planes: 0..2 : negative comp. 3..5 : positive component
    vector center;				  // center -> lightsource
    lightbuf_pixel *find_pixel(ray *r)const;	  // find the pixel associated with this direction.

public:
    void print() const;
    lightbuf(vector);
    bool test(ray *r)const;
};
/**   lightbuf contains 6 faces of a cube centered in light.org */
#endif
