#ifndef COLOR_HH
#define COLOR_HH
#include "color.hh"

pixel_data *search_pixels(const char *fn);
void add_pixels(pixel_data *);


/// pixelbased input files: return color for each 2D point
class pixel_data
{
    friend pixel_data *search_pixels(const char *fn);
    char   *name;
protected:
    virtual byte_color get_byte_color(int,int) const = 0;

public:
    int     imagex,
            imagey;             /* resolution */
    int     refs;
    void print() const;
    virtual ~pixel_data();
    pixel_data(const char *s);
    
    color interpolate(flt,flt) const;
    color get_color(flt,flt) const;    
};
/// return color based on a palette
struct colmapped_data: pixel_data {
    byte_color   *pal;
    byte *data;               /* the data */
    int     palsize;    
    byte_color get_byte_color(int,int) const;
    colmapped_data (const char *s);
    ~colmapped_data () { delete[] pal; delete[] data; }
};
/// return color based on truecolor info
struct truecol_data : pixel_data {
    byte_color *data;               /* the data */
    void print() const;
    byte_color get_byte_color(int,int) const;
    truecol_data (const char *s);
    ~truecol_data () { delete[] data; }
};

#endif
