/*
 * poly.h -- routines for manipulation of polynomials in one var
 *
 * (c) 1993, 1996 Han-Wen Nienhuys
 */

#ifndef POLY_H
#define POLY_H
#include "ray.hh"

/// structure for a polynomial in one var. 
struct Poly
{
    /// degree of polynomial 
    int     ord;     

    /// coefficients 
    flt    coef[MAX_ORDER + 1];
    
    int degree() const { return ord; }
    flt &lc()  { return coef[ord]; }
    flt lc() const { return coef[ord]; }
    void print() const ;	
    flt eval(flt) const ;
    void print_sols(const flt *, int) const ;
    void check_sols(const flt *, int) const ;
    /// is #x# a zero? print warning if not.
    void  check_sol(flt x) const;
    void multiply(const Poly & p1, const Poly & p2);
    void power(int exponent, const Poly & src);

    /// chop low coefficients
    void clean();

    /// eliminate #x#  close to  zero
    void real_clean();

    void add(const Poly & p1, const Poly & p2);
    void scalarmultiply(flt fact);
    void operator *= (flt f) { scalarmultiply(f); }
    void operator /= (flt f) { scalarmultiply(1/f); }
    void subtract(const Poly & p1, const Poly & p2);
    void negate(const Poly & src);
    
    /// take the derivative
    void differentiate();
    int mod(const Poly &u, const Poly &v);
    int solve_rt(flt *, bool);
    int sturm_solve_rt(flt *, bool);
    Poly() { ord = 0; coef[0] = 0; }
    Poly(flt a,flt b=0) {
	coef[0] = a;
	coef[1] = b;
	ord = bool(b);
    }
    void debug_clean();
};


int solve_quadric(const flt c[3], flt s[2]);
int solve_rt_quadric(const flt * coeffs, flt * roots);

#endif

